(* syntax abstraite *)

Require Arith.
Require Compare_dec.
Require Peano_dec.
Require PolyList.
Require Bool.
Require Extraction.

Inductive color:Set:=
   R:color
  |G:color
  |Bc:color
  |C:color
  |M:color
  |Y:color
  |K:color
  |W:color.

Inductive space:Set:=
   CR :space
  |LF :space 
  |TAB:space
  |SPC:space.

Inductive document:Set:=
   B    :document->document
  |EM   :document->document
  |I    :document->document
  |PL   :document->document
  |St   :document->document
  |TT   :document->document
  |U    :document->document
  |Size :nat->document->document
  |Color:color->document->document
  |Text :nat->document
  |Space:space->document
  |Seq  :document->document->document
  |Nil  :document.

Inductive car:Set:=
   Print:nat->car
  |Spc  :space->car.

Record context:Set:= mkcontext
{ b     :bool;
  em    :bool;
  i     :bool;
  s     :bool;
  tt    :bool;
  u     :nat;
  sizec :nat;
  colorc:color
}.

Inductive option [A:Set]:Set:=
  None:(option A)
 |Some:A->(option A).

(* decidabilite *)

Lemma bool_dec:(A,B:bool){A=B}+{~A=B}.
Decide Equality.
Save.

Lemma color_dec:(A,B:color){A=B}+{~A=B}.
Decide Equality.
Save.

Lemma context_dec:(A,B:context){A=B}+{~A=B}.
Decide Equality;Apply color_dec Orelse Apply bool_dec Orelse Apply eq_nat_dec.
Save.

(* Lemmes sur les listes *)

Lemma eq_rev:(A:Set)(e,f:(list A))e=f->(rev e)=(rev f). 
Intros;Rewrite H;Trivial.
Save.

Lemma list_dec:(A:Set)(l:(list A))
   {a:A & {tl:(list A)|l=(cons a tl)}}+{l=(nil A)}.
Induction l.
Right;Reflexivity.
Intros;Left;Split with a;Split with l0;Reflexivity.
Save.

(* meaning *)

Fixpoint meaning [c:context;prev:(option context);
                  doc:document]:(list (car*context)):=
  Cases doc of
    (B d)      => (meaning  
    (mkcontext true (em c) (i c) (s c) (tt c) (u c) (sizec c) (colorc c)) 
                  prev d)
   |(EM d)     => if (s c) then (meaning c prev d) 
                  else (meaning  
    (mkcontext (b c) (negb (em c)) (i c) (s c) (tt c) (u c) 
               (sizec c) (colorc c)) prev d)
   |(I d)      => (meaning  
    (mkcontext (b c) (em c) true (s c) (tt c) (u c) (sizec c) (colorc c)) 
                   prev d)
   |(PL d)     => (meaning  
    (mkcontext false false false false false (0) (sizec c) (colorc c)) 
                   prev d)
   |(St d)     => (meaning  
    (mkcontext (b c) false (i c) true (tt c) (u c) (sizec c) (colorc c)) 
                   prev d)
   |(TT d)     => (meaning  
    (mkcontext (b c) (em c) (i c) (s c) true (u c) (sizec c) (colorc c)) 
                   prev d)
   |(U d)      => 
      (Cases (le_lt_dec (u c) (2)) of
         (left _)  =>(meaning  
  (mkcontext (b c) (em c) (i c) (s c) (tt c) (S (u c)) (sizec c) (colorc c)) 
                     prev d)
        |   _      =>(meaning  c prev d)
       end)
   |(Size n d) =>(meaning  
    (mkcontext (b c) (em c) (i c) (s c) (tt c) (u c) n (colorc c)) prev d)
   |(Color n d)=>(meaning  
    (mkcontext (b c) (em c) (i c) (s c) (tt c) (u c) (sizec c) n) prev d)
   |(Text n)   =>(cons ((Print n),c) (nil (car*context)))
   |(Space sp) =>
      [space_context:=
        (Cases (eq_nat_dec (u c) (0)) of
           (left _) =>
      (mkcontext false false false false (tt c) (u c) (sizec c) W)
          | _       =>  
      (mkcontext false false false false (tt c) (u c) (sizec c) (colorc c))
         end)] 
  (if (tt c) then (cons ((Spc sp),space_context) (nil (car*context)))
   else (Cases prev of
          None => (cons ((Spc sp),space_context) (nil (car*context)))
         |(Some ct) => 
            (Cases (context_dec ct space_context) of
               (left _)=>(nil (car*context))
              |  _     =>(cons ((Spc sp),space_context) (nil (car*context)))
             end)
         end))
   |(Seq d1 d2)=>[list_d1:=(meaning c prev d1)]
      (Cases (rev list_d1) of
         nil                   =>(meaning c prev d2) 
        |(cons ((Spc _),ct) t) =>(app list_d1 (meaning c (Some context ct) d2))
        | _                    =>(app list_d1 (meaning c (None context) d2))
       end)
   |  Nil      =>(nil (car*context))
  end.

(* il exist un P equivalent *)

Theorem existP:(P:document)(state:context)(co:(option context))
  {P':document|(meaning state co P')=(meaning state co P)}.
Induction P;Intros.
(* cas B *)
Case (bool_dec (b state) true);Intro.
Elim (H state co);Clear H;Intros x H;Split with x;Simpl;Rewrite <- e;
 Rewrite H;Elim state;Intro;Auto.
Elim (H (mkcontext true (em state) (i state) (s state) (tt state) (u state) 
    (sizec state) (colorc state)) co);Clear H;Intros x H;Split with (B x);
 Simpl;Assumption.
(* cas EM *)
Case (bool_dec (s state) true);Intro.
Elim (H state co);Clear H;Intros x H;Split with x;Simpl;Rewrite e;Assumption.
Elim (H (mkcontext (b state) (negb (em state)) (i state) (s state) 
 (tt state) (u state) (sizec state) (colorc state)) co);Clear H;Intro x;
Elim x;
  [Intros;Idtac|Intros;Idtac|Intros;Idtac|Intros;Idtac|
   Intros;Idtac|Intros;Idtac|Intros;Idtac|Intros;Idtac|
   Intros;Idtac|Intros;Idtac|Intros s0 p;Idtac|Intros;Idtac|
   Intros;Idtac].
Split with (EM (B d0));Simpl;Generalize (not_true_is_false (s state) n);
 Intro;Rewrite H0;Rewrite <-H0;Assumption.
Split with d0;Simpl in p;Generalize (not_true_is_false (s state) n);
 Intro;Rewrite H0 in p;Rewrite negb_elim in p;
 Simpl;Rewrite H0;Rewrite <-p;Rewrite <-H0;Elim state;Auto.
Split with (EM (I d0));Simpl;Generalize (not_true_is_false (s state) n);
 Intro;Rewrite H0;Rewrite <-H0;Assumption.
Split with (EM (PL d0));Simpl;Generalize (not_true_is_false (s state) n);
 Intro;Rewrite H0;Rewrite <-H0;Rewrite <-p;Rewrite H0;Trivial.
Split with (EM (St d0));Simpl;Generalize (not_true_is_false (s state) n);
 Intro;Rewrite H0;Rewrite <-H0;Rewrite <-p;Simpl;Rewrite <-H0;Trivial.
Split with (EM (TT d0));Simpl;Generalize (not_true_is_false (s state) n);
 Intro;Rewrite H0;Rewrite <-H0;Assumption.
Split with (EM (U d0));Simpl;Generalize (not_true_is_false (s state) n);
 Intro;Rewrite H0;Rewrite <-H0;Assumption.
Split with (EM (Size n0 d0));Simpl;Generalize (not_true_is_false (s state) n);
 Intro;Rewrite H0;Rewrite <-H0;Assumption.
Split with (EM (Color c d0));Simpl;Generalize (not_true_is_false (s state) n);
 Intro;Rewrite H0;Rewrite <-H0;Assumption.
Split with (EM (Text n0));Simpl;Generalize (not_true_is_false (s state) n);
 Intro H0;Rewrite H0;Rewrite <-H0;Assumption.
Split with (EM (Space s0));Cut [l:=(Space s0)]
 (meaning state co (EM l))=(meaning state co (EM d));Auto.
Unfold 1 meaning;Cbv Iota;Fold meaning;
 Generalize (not_true_is_false (s state) n);
 Intro H0;Rewrite H0;Rewrite <-H0;Intro;Cbv Delta [l];
 Rewrite p;Simpl;Rewrite H0;Rewrite <-H0;Trivial.
Split with (EM (Seq d0 d1));Simpl;Generalize (not_true_is_false (s state) n);
 Intro H1;Rewrite H1;Rewrite <-H1;Assumption.
Split with (EM Nil);Simpl;Generalize (not_true_is_false (s state) n);
 Intro H1;Rewrite H1;Rewrite <-H1;Assumption.
(* cas I *)
Case (bool_dec (i state) true);Intro.
Elim (H state co);Clear H;Intros x H;Split with x;Simpl;Rewrite <- e;
 Rewrite H;Elim state;Intro;Auto.
Elim (H (mkcontext (b state) (em state) true (s state) (tt state) (u state) 
    (sizec state) (colorc state)) co);Clear H;Intros x H;Split with (I x);
 Simpl;Assumption.
(* cas PL *)
Elim (H (mkcontext false false false false false (0) (sizec state) 
  (colorc state)) co);Clear H;Intros x H;Split with (PL x);
 Simpl;Assumption.
(* cas St *)
Elim (H (mkcontext (b state) false (i state) true (tt state) (u state) 
    (sizec state) (colorc state)) co);Clear H;Intros x H;Split with (St x);
 Simpl;Assumption.
(* cas TT *)
Case (bool_dec (tt state) true);Intro.
Elim (H state co);Clear H;Intros x H;Split with x;Simpl;Rewrite <- e;
 Rewrite H;Elim state;Intro;Auto.
Elim (H (mkcontext (b state) (em state) (i state) (s state) true (u state) 
    (sizec state) (colorc state)) co);Clear H;Intros x H;Split with (TT x);
 Simpl;Assumption.
(* cas U *)
Case (le_lt_dec (u state) (2));Intro.
Elim (H (mkcontext (b state) (em state) (i state) (s state) (tt state) 
     (S (u state)) (sizec state) (colorc state)) co);Clear H;Intros x H;
 Split with (U x);Simpl;Case (le_lt_dec (u state) (2));Intro.
Assumption.
Generalize (le_not_lt (u state) (2) l);Intro;ElimType False;Auto.
Elim (H state co);Clear H;Intros x H;Split with x;Simpl;
 Case (le_lt_dec (u state) (2));Intro.
Generalize (le_not_lt (u state) (2) l0);Intro;ElimType False;Auto.
Assumption.
(* cas Size *)
Elim (H (mkcontext (b state) (em state) (i state) (s state) (tt state) 
  (u state) n (colorc state)) co);Clear H;Intros x H;Split with (Size n x);
 Simpl;Assumption.
(* cas Color *)
Elim (H (mkcontext (b state) (em state) (i state) (s state) (tt state) 
  (u state) (sizec state) c) co);Clear H;Intros x H;Split with (Color c x);
 Simpl;Assumption.
(* cas Text *)
Split with (Text n);Auto.
(* cas Space *)
Case (bool_dec (tt state) true);Intro.
Split with (Space s);Auto.
Elim co.
Split with (Space s);Auto.
Intro;Generalize (not_true_is_false (tt state) n);Intro;
  Simpl;Rewrite H;
 Case (context_dec a
             Cases (eq_nat_dec (u state) (0)) of
               (left _) => 
                (mkcontext false false false false false (u state)
                  (sizec state) W)
             | (right _) => 
                (mkcontext false false false false false (u state)
                  (sizec state) (colorc state))
             end);Intro.
Split with Nil;Simpl;Auto.
Split with (Space s);Simpl;Rewrite H;
 Case (context_dec a
        Cases (eq_nat_dec (u state) (0)) of
          (left _) => 
           (mkcontext false false false false false (u state)
             (sizec state) W)
        | (right _) => 
           (mkcontext false false false false false (u state)
             (sizec state) (colorc state))
        end);Intro.
ElimType False;Auto.
Trivial.
(* cas Seq *)
Elim (H state co);Clear H;Intros x H;Simpl;Rewrite <-H;
 Elim (list_dec car*context (rev (meaning state co x))); Intro.
Elim a; Clear a; Intros;Elim p; Clear p; Intros;Generalize p;Elim x0;
 Induction a; Intros;Rewrite p0.
Elim (H0 state (None context)); Clear H0; Intros y H0;
 Split with (Seq x y);Simpl;Rewrite p0;Rewrite H0;Trivial.
Elim (H0 state (Some context b)); Clear H0; Intros y H0;
 Split with (Seq x y);Simpl;Rewrite p0;Rewrite H0;Trivial.
Rewrite b;Elim (H0 state co); Clear H0; Intros y H0; Split with y; Simpl;
  Assumption.
(* cas Nil *)
Split with Nil;Auto.
Save.

(* extraction *)

Extraction "equiv.ml" existP.
