(* printer.ml *)

open Syntax

let string_of_color = function
  | R -> "r"
  | G -> "g"
  | Bc -> "b"
  | C -> "c"
  | M -> "m"
  | Y -> "y"
  | K -> "k"
  | W -> "w"

let string_of_space = function
  | CR -> "\013"
  | LF -> "\010"
  | TAB -> "\t"
  | SPC -> " "

let rec string_of_ast = function
  | B doc -> "<B>"^(string_of_ast doc)^"</B>"
  | EM doc -> "<EM>"^(string_of_ast doc)^"</EM>"
  | I doc -> "<I>"^(string_of_ast doc)^"</I>"
  | PL doc -> "<PL>"^(string_of_ast doc)^"</PL>"
  | St doc -> "<S>"^(string_of_ast doc)^"</S>"
  | TT doc -> "<TT>"^(string_of_ast doc)^"</TT>"
  | U doc -> "<U>"^(string_of_ast doc)^"</U>"
  | Size (n,doc) ->
    let ns = string_of_int n in
    "<"^ns^">"^(string_of_ast doc)^"</"^ns^">"
  | Color (c,doc) ->
    let cs = string_of_color c in
    "<"^cs^">"^(string_of_ast doc)^"</"^cs^">"
  | Text s -> s
  | Space sp -> string_of_space sp
  | Seq (a,b) -> (string_of_ast a)^(string_of_ast b)
  | Nil -> ""

let print_ast ast = print_endline (string_of_ast ast)
